#include "hdf5.h"
#include "stdlib.h"

int
main (int argc, char **argv)
{

int i;
int mpi_rank, mpi_size;
int rank = 2;
int *data;             /* pointer to data buffer to write */

hid_t file_id;         /* file identifier */
hid_t dset_id;         /* dataset identifiers */
hid_t filespace;       /* file dataspace identifiers */
hid_t filespace1;      /* file dataspace identifiers (hyperslab) */
hid_t memspace;        /* memory dataspace identifiers */
hid_t plist_id;        /* property list identifier */
hsize_t dimsf[rank];   /* global dimensions */
hsize_t count[rank];   /* local dimension (hyperslab) */
hsize_t offset[rank];  /* local offset (hyperslab) */
herr_t status;


char H5FILE_NAME[128]; /* filename */

/* Initialize MPI */
MPI_Init(&argc, &argv);
MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);

/* setting filename */
sprintf(H5FILE_NAME,"%s.h5","RUN/my_third_parallel_file");
printf("I'm task %d of %d \n",mpi_rank,mpi_size);
if (mpi_rank==0) printf("creating h5 file %s...", H5FILE_NAME);
if (mpi_rank==0) printf("...contigous hyperslab\n");

/* set global size */
dimsf[0] = 10;
dimsf[1] = 8;

/* Each task defines dataset in memory and writes it to the hyperslab in the file */
count[0] = dimsf[0]/mpi_size;
count[1] = dimsf[1];
//printf("I'm task %d of %d \n, %d, %d",mpi_rank,mpi_size,count[0],count[1]);
memspace = H5Screate_simple(rank, count, NULL);

/* set offset for each task */
offset[0] = mpi_rank * count[0];
offset[1] = 0;

/* Initialize data buffer  */
data = (int *) malloc(sizeof(int)*count[0]*count[1]);
for (i=0; i < count[0]*count[1]; i++) {
     data[i] = mpi_rank*1000 + i;
}

/* Set up file access property list for MPI-IO access */
plist_id = H5Pcreate(H5P_FILE_ACCESS);
H5Pset_fapl_mpio(plist_id, MPI_COMM_WORLD, MPI_INFO_NULL);

file_id = H5Fcreate(H5FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT, plist_id);

/* release property list identifier */
H5Pclose(plist_id);

/* Create the dataspace for the dataset.  */
filespace = H5Screate_simple(rank, dimsf, NULL);

/* Create the dataset with default properties collective. */
//dset_id = H5Dcreate(file_id, "dataset1", H5T_NATIVE_INT, filespace, H5P_DEFAULT);
// 1.8 API
dset_id = H5Dcreate(file_id, "dataset1", H5T_NATIVE_INT, filespace, H5P_DEFAULT,H5P_DEFAULT,H5P_DEFAULT);   

/* Select hyperslab in the file  */
filespace1 = H5Dget_space(dset_id);
H5Sselect_hyperslab(filespace1, H5S_SELECT_SET, offset, NULL, count, NULL);

/* Create property list for collective dataset write */ 
plist_id = H5Pcreate(H5P_DATASET_XFER);
// H5Pset_dxpl_mpio(plist_id, H5FD_MPIO_COLLECTIVE);
H5Pset_dxpl_mpio(plist_id, H5FD_MPIO_INDEPENDENT);

/* Write.... */
status = H5Dwrite(dset_id, H5T_NATIVE_INT, memspace, filespace1, plist_id, data);

/* close dataset */
H5Dclose(dset_id);

/* close */
H5Sclose(memspace);
H5Sclose(filespace);
H5Sclose(filespace1);

/* Close the file. */
H5Fclose(file_id);
if (mpi_rank==0) printf("...all done\n");
MPI_Finalize();

}


