!
program ex9
!
      use HDF5
      implicit none
!
      character(len=20), parameter :: file_name="RUN/my_5th_file.h5"
      integer(hid_t) :: file_id       !! file identifier
      integer(hid_t) :: group_id      !! group identifier
      integer(hid_t) :: groupA_id     !! sub-group identifier
      integer(hid_t) :: dataset_id    !! dasaset identifier
      integer(hid_t) :: dataspace_id  !! dataspace identifier
      integer(hid_t) :: str_type      !! string datatype
      integer(hsize_t) :: dims(1)
      integer        :: hdferr
      integer*8, parameter :: str_len = 3
      character(len=str_len), dimension(7) :: days  = (/"Mon","Tue","Wed","Thu","Fri","Sat","Sun"/)
!
! set size...
      dims(1) = 7
!
      write(6,*) "writing h5 file ", file_name, " with group and dataset"
!
! open the fortran interface for hdf5   
      call h5open_f(hdferr);

! create a custom datatype (fixed length string)
      call h5tcopy_f(H5T_FORTRAN_S1, str_type, hdferr)
      call H5Tset_size_f(str_type, str_len, hdferr)

! create the file using default properties.
      call h5fcreate_f(file_name, H5F_ACC_TRUNC_F, file_id, hdferr, &
                       H5P_DEFAULT_F, H5P_DEFAULT_F)

! create a group named "/Week" in the file. */
      call h5gcreate_f(file_id, "Week", group_id, hdferr, &
                       OBJECT_NAMELEN_DEFAULT_F, H5P_DEFAULT_F, &
                       H5P_DEFAULT_F, H5P_DEFAULT_F)

! create sub-group "Days" in group "Week" using absolute name. */
      call h5gcreate_f(file_id, "Week/Days", groupA_id, hdferr, &
                       OBJECT_NAMELEN_DEFAULT_F, H5P_DEFAULT_F, &
                       H5P_DEFAULT_F, H5P_DEFAULT_F)

! create simple dataspace 
      call h5screate_simple_f(1, dims, dataspace_id, hdferr, dims)

! create the dataset 
      call h5dcreate_f(file_id, "dset", str_type, &
                       dataspace_id, dataset_id, hdferr, H5P_DEFAULT_F,  & 
                       H5P_DEFAULT_F, H5P_DEFAULT_F)
! write the dataset
      call h5dwrite_f(dataset_id, str_type, days, dims, hdferr)

! close dataspace & dataset & datatype
      call h5sclose_f(dataspace_id, hdferr)
      call h5dclose_f(dataset_id, hdferr)
      call h5tclose_f(str_type, hdferr)

! close the group
      call h5gclose_f(group_id, hdferr)
      call h5gclose_f(groupA_id, hdferr)

! close the file
      call h5fclose_f(file_id, hdferr)

! close the interface
      call h5close_f(hdferr)
      write(6,*) ".... done"

end program ex9
